g! /usr/bin/env python
## -*- coding: utf-8 -*-
## (C) 2012: Hans Georg Schaathun <georg@schaathun.net> 

"""
Support Vector Machines with Grid Search.

This provides a wrapper for the LibSvm, where a grid search is
performed as an integral part of training.  Thus gamma and C do
not have to be specified at initialisation time; instead search
ranges are provided and the training method finds the optimal
values within the range.
"""

from . import Classifier

class SVM(Classifier):
   def __init__(self,k,crange=( -5,  15,  2),grange=(  3, -15, -2),
         nworker = 1, gnuplot=None, verbosity=0, **kw):

	 **kw):
	 # svm_type=’c_svc’, kernel_type=’rbf’, degree=3,
	 # gamma=0.001, coef0=0, C=1, nu=0.5, eps=0.001,
	 # p=0.1, cache_size=100, shrinking=True, probability=False,
	 # weight={})
      self.k = k
      self.nworker = nworker
      self.gnuplot = gnuplot
      self.verbosity = verbosity
      self.crange = crange
      self.grange = grange
      self.libsvm = None
      self.param = kw
   def learn(self,x,y):
      if ( self.param["svm_type"] == "c_svc" and 
         self.param["kernel_type"] != "linear" ):
         (db,(best_c1, best_g1),(best_c, best_g), best_rate) = \
	       gridsearch(x,y,k,self.crange,self.grange,
	           self.nworker,self.gnuplot,self.verbosity)
	 self.param["gamma"] = best_g
	 self.param["C"] = best_c
      self.libsvm = LibSvm( **self.param )
      return self.libsvm.learn(x,y)
   def pred(self,*a,**kw):
      return self.libsvm.pred(*a,**kw)
   def pred_probability(self,*a,**kw):
      return self.libsvm.pred_probability(*a,**kw)
   def pred_values(self,*a,**kw):
      return self.libsvm.pred_values(*a,**kw)
   def labels(self,*a,**kw):
      return self.libsvm.labels(*a,**kw)
   def nclasses(self,*a,**kw):
      return self.libsvm.nclasses(*a,**kw)
   def nsv(self,*a,**kw):
      return self.libsvm.nsv(*a,**kw)
   def label_nsv(self,*a,**kw):
      return self.libsvm.label_nsv(*a,**kw)
   @classmethod
   def load_model(cls,*a,**kw):
      C = cls()
      C.libsvm = LibSvm.load_model(*a,**kw)
      return C
   def save_model(self,*a,**kw):
      return self.libsvm.save_model(*a,**kw)
